#! /bin/sh
#
# Keyboard Driver Patch release 1.30
# by ITO Takayuki <yuki@euc.jp>
# http://euc.jp/beos/beware.en.html
#
# This script modifies BeOS R4.5.x/R5 keyboard drivers
# to enable some keys.

install_pc ()
{
	[ "x$DIR1" = x ] && DIR1=/system/add-ons/input_server/devices
	FILE1=keyboard
	[ "x$DIR2" = x ] && DIR2=/system/add-ons/kernel/drivers/bin
	FILE2=hid
	
	# check original driver
	
	if [ ! -f $DIR1/$FILE1 ]
	then
		alert --stop "Original driver $DIR1/$FILE1 not present."
		exit 1
	fi
	if [ ! -f $DIR2/$FILE2 ]
	then
		alert --stop "Original driver $DIR2/$FILE2 not present."
		exit 1
	fi
	
	if [ -f $DIR1/$FILE1.gz ]
	then
		alert --warning "Patch is already applied for $DIR1/$FILE1."
	else
		./bsubst < $DIR1/$FILE1 \
		"6e 00 00 00 00 00 00 00 00 6d 00 6c 00 6a 00 00" \
		"6e 00 00 6b 00 00 00 00 00 6d 00 6c 00 6a 70 00" \
		| ./bsubst \
		"6e 00 00 6b 00 00 00 00 00 6d 00 6c 00 6a 00 00" \
		"6e 00 00 6b 00 00 00 00 00 6d 00 6c 00 6a 70 00" \
		> $DIR1/$FILE1.new
		cmp -s $DIR1/$FILE1 $DIR1/$FILE1.new
		if [ $? -eq 0 ]
		then
			alert --warning "Patch is not applicable. Nothing altered."
			rm $DIR1/$FILE1.new
		else
			gzip -c $DIR1/$FILE1 > $DIR1/$FILE1.gz
			mv $DIR1/$FILE1.new $DIR1/$FILE1
		fi
	fi
	
	if [ -f $DIR2/$FILE2.gz ]
	then
		alert --warning "Patch is already applied for $DIR2/$FILE2."
	else
		./bsubst < $DIR2/$FILE2 \
		"32 33 00 45 46 11 53 54  55 3b 02 03 04 05 06 07" \
		"32 33 33 45 46 11 53 54  55 3b 02 03 04 05 06 07" \
		| ./bsubst \
		"47 83 ff 06" \
		"47 83 ff 07" \
		| ./bsubst \
"38 39 64 65 00 68 00 00  00 00 00 00 00 00 00 00 \
 00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00 \
 00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00 \
 00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00" \
"38 39 64 65 69 68 71 6f  00 00 00 00 00 00 00 00 \
 00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00 \
 00 00 00 00 00 70 00 6b  6e 6a 6d 6c 70 00 00 00 \
 72 73 00 00 00 00 00 00  00 00 00 00 00 00 00 00" \
		> $DIR2/$FILE2.new
		cmp -s $DIR2/$FILE2 $DIR2/$FILE2.new
		if [ $? -eq 0 ]
		then
			alert --warning "Patch is not applicable. Nothing altered."
			rm $DIR2/$FILE2.new
		else
			gzip -c $DIR2/$FILE2 > $DIR2/$FILE2.gz
			mv $DIR2/$FILE2.new $DIR2/$FILE2
		fi
	fi
	
	alert --info "All done.
	
In case of trouble you can recover the original driver as follows:

$ cd $DIR1
$ gunzip $FILE1.gz
$ cd $DIR2
$ gunzip $FILE2.gz
"
}

install_mac ()
{
	[ "x$DIR1" = x ] && DIR1=/system
	FILE1=kernel_mac
	
	# check original driver
	
	if [ ! -f $DIR1/$FILE1 ]
	then
		alert --stop "Original kernel $DIR1/$FILE1 not present."
		exit 1
	fi
	
	if [ -f $DIR1/$FILE1.gz ]
	then
		alert --warning "Patch is already applied for $DIR1/$FILE1."
	else
		./bsubst < $DIR1/$FILE1 \
"68 22 68 68 68 23 5b 68  25 68 68 6a 64 58 59 5a \
 48 49 4a 37 68 38 39 68  68 68 06 07 08 04 09 0a \
 68 0c 68 0e 68 0f 68 0b  68 0d 68 10 1f 20 21 34 \
 05 35 03 36 02 56 67 60  68 6b 44 69 64 6e 27 74" \
"68 22 68 68 68 23 5b 68  25 68 68 6f 64 58 59 5a \
 48 49 4a 37 68 38 39 6a  6b 70 06 07 08 04 09 0a \
 73 0c 72 0e 74 0f 68 0b  68 0d 68 10 1f 20 21 34 \
 05 35 03 36 02 56 67 60  68 71 44 69 64 6e 27 74" \
		> $DIR1/$FILE1.new
		cmp -s $DIR1/$FILE1 $DIR1/$FILE1.new
		if [ $? -eq 0 ]
		then
			alert --warning "Patch is not applicable. Nothing altered."
			rm $DIR1/$FILE1.new
		else
			mv $DIR1/$FILE1 $DIR1/$FILE1.orig
			mv $DIR1/$FILE1.new $DIR1/$FILE1
		fi
	fi
	
	alert --info "All done.

In case of trouble you can recover the original kernel as follows:

$ cd $DIR1
$ mv $FILE1.orig $FILE1
"
}

#
# main
#

cd `dirname $0`

alert --warning "This script will apply patch to the keyboard drivers to enable some keys." OK Quit > /dev/null
if [ $? -eq 1 ]
then
	alert --stop "Aborted."
	exit 1
fi

if [ ! -x ./bsubst ]
then
	if [ -f bsubst.c ]
	then
		cc -o bsubst bsubst.c
	else
		alert --stop "bsubst is missing."
		exit 1
	fi
fi

case `uname -m` in
BePC)
	install_pc
	;;
BeMac)
	install_mac
	;;
BeBox)
	alert --stop "Not applicable to BeBox."
	exit 1
	;;
esac

exit 0
